/*
 * ProjectsDAO.java
 *
 * Created on 05 June 2006, 16:43
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package jim.sums3.process;

import formdef.plugin.util.FormUtils;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import jim.common.JimFatalException;
import jim.sums3.general.forms.ProjectChangeForm;
import jim.sums3.pojo.FinalProject;
import jim.sums3.pojo.FinalProjectHome;
import jim.sums3.pojo.MarkerType;
import jim.sums3.pojo.Person;
import jim.util.hiber.HibernateUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.FetchMode;
import org.hibernate.Transaction;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Restrictions;

/**
 *
 * @author briggsj
 */
public class FinalProjectDAO extends FinalProjectHome {
    
    private static final Log log = LogFactory.getLog(FinalProjectDAO.class);
    
    /** Creates a new instance of ProjectsDAO */
    public FinalProjectDAO() {
    }
    
    public List<FinalProject> findAllProjects() {
        log.debug("finding all projects");
        try {
            List<FinalProject> results = (List<FinalProject>) HibernateUtil.getSession()
            .createCriteria("jim.sums3.pojo.FinalProject")
            .setFetchMode("student", FetchMode.JOIN)
            .list();
            log.debug("finding all projects successful, result size: " + results.size());
            return results;
        } catch (RuntimeException re) {
            log.error("finding all projects failed", re);
            throw re;
        }
    }
    
    public List<FinalProject> findAllProjectsSorted() {
        List <FinalProject> results = findAllProjects();
        Collections.sort(results);
        return results;
    }
    
    static private Map<Character, MarkerType> mtypes = new HashMap<Character, MarkerType>();
    {
        MarkerTypeDAO dao = new MarkerTypeDAO();
        mtypes.put('S', dao.supervisor());
        mtypes.put('M', dao.moderator());
        mtypes.put('3', dao.thirdMarker());
        mtypes.put('E', dao.externalMarker());
        
    }
    
    public List<FinalProject> findByMarker(Person p, String markers) {
        log.debug("finding projects by marker (" + markers.toString() + ") for " + p.getName());
        try {
            Criteria c = HibernateUtil.getSession().createCriteria("jim.sums3.pojo.FinalProject");
            Criteria pc = c.createCriteria("markerCapacities");
            
            if (p != null) {
                pc.add(Restrictions.eq("person", p));
            }
            
            Disjunction d = Restrictions.disjunction();
            for (int i=0; i<markers.length(); i++) {
                d.add(Restrictions.eq("markerType", mtypes.get(markers.charAt(i))));
            }
            pc.add(d);
            
            List <FinalProject> results = (List<FinalProject>)c.list();
            log.debug("finding projects successful, result size: " + results.size());
            return results;
        } catch (RuntimeException re) {
            log.error("finding projects failed", re);
            throw re;
        }
    }
        
    public FinalProject changeProject(ProjectChangeForm form, Person user) throws JimFatalException {
        log.debug("changing project " + form.getProjectId());
        HibernateUtil.beginTransaction();
        FinalProject project = findById(form.getProjectId());
        if (project == null) {
            log.error("Canot find project with ID=" + form.getProjectId());
            throw new JimFatalException("Cannot find project with ID=" + form.getProjectId());
        }
        AuditTrailDAO audit = new AuditTrailDAO();
        
        String newTitle = (String)form.getProject().get("projectTitle");
        if (!project.getProjectTitle().equals(newTitle)) {
            audit.log(project, "projectTitle", project.getProjectTitle(), newTitle, form.getReason(), user);
            project.setProjectTitle(newTitle);
        }
        HibernateUtil.commitTransaction();
        return project;
    }
}
